@@ -57,9 +57,6 @@ QStringList TagLibExtractor::mimetypes()
     // WAV
     types << QLatin1String("audio/wav");
 
-    // MP4
-    types << QLatin1String("video/mp4");
-
     // AIFF
     types << QLatin1String("audio/x-aiff");
 
@@ -97,15 +94,11 @@ Nepomuk2::SimpleResourceGraph TagLibExtractor::extract(const QUrl& resUri, const
             fileRes.addProperty( NMM::genre(), genre );
         }
 
-        // TODO: Split artists
-        QString artists = QString::fromUtf8( tags->artist().toCString( true ) );
-        if( !artists.isEmpty() ) {
-            SimpleResource artist;
-            artist.addType( NCO::Contact() );
-            artist.setProperty( NCO::fullname(), artists );
-
-            fileRes.setProperty( NMM::performer(), artist );
-            graph << artist;
+        QString artistString = QString::fromUtf8( tags->artist().toCString( true ) );
+        QList< SimpleResource > artists = contactsFromString( artistString );
+        foreach( const SimpleResource& res, artists ) {
+            fileRes.addProperty( NMM::performer(), res );
+            graph << res;
         }
 
         QString album = QString::fromUtf8( tags->album().toCString( true ) );
@@ -162,4 +155,4 @@ Nepomuk2::SimpleResourceGraph TagLibExtractor::extract(const QUrl& resUri, const
 
 }
 
-NEPOMUK_EXPORT_EXTRACTOR( Nepomuk2::TagLibExtractor, "nepomuktaglibextextractor" )
\ No newline at end of file
+NEPOMUK_EXPORT_EXTRACTOR( Nepomuk2::TagLibExtractor, "nepomuktaglibextextractor" )
