@@ -1,5 +1,6 @@
 /* This file is part of the KDE Project
    Copyright (c) 2007-2011 Sebastian Trueg <trueg@kde.org>
+   Copyright (c) 2012-2013 Vishesh Handa <me@vhanda.in>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -24,6 +25,7 @@
 #include "removablemediacache.h"
 #include "fileindexerconfig.h"
 #include "activefilequeue.h"
+#include "removablemediadatamigrator.h"
 
 #ifdef BUILD_KINOTIFY
 #include "kinotify.h"
@@ -342,25 +344,56 @@ void Nepomuk2::FileWatch::slotDeviceMounted(const Nepomuk2::RemovableMediaCache:
     // tell the file indexer to update the newly mounted device
     //
     KConfig fileIndexerConfig( "nepomukstrigirc" );
+    const QByteArray devGroupName( "Device-" + entry->url().toUtf8() );
     int index = 0;
+    // Old-format -> port to new format
     if(fileIndexerConfig.group("Devices").hasKey(entry->url())) {
-        index = fileIndexerConfig.group("Devices").readEntry(entry->url(), false) ? 1 : -1;
+        KConfigGroup devicesGroup = fileIndexerConfig.group( "Devices" );
+        index = devicesGroup.readEntry(entry->url(), false) ? 1 : -1;
+        devicesGroup.deleteEntry( entry->url() );
+
+        KConfigGroup devGroup = fileIndexerConfig.group( devGroupName );
+        devGroup.writeEntry( "mount path", entry->mountPath() );
+        if( index == 1 )
+            devGroup.writeEntry( "folders", QStringList() << QLatin1String("/") );
+        else if( index == -1 )
+            devGroup.writeEntry( "exclude folders", QStringList() << QLatin1String("/") );
+
+        fileIndexerConfig.sync();
+
+        // Migrate the existing filex data
+        RemovableMediaDataMigrator* migrator = new RemovableMediaDataMigrator( entry );
+        QThreadPool::globalInstance()->start( migrator );
+    }
+
+    // Already exists
+    else if( fileIndexerConfig.hasGroup( devGroupName ) ) {
+        // Inform the indexer to start the indexing process
+
+        org::kde::nepomuk::FileIndexer fileIndexer( "org.kde.nepomuk.services.nepomukfileindexer", "/nepomukfileindexer", QDBusConnection::sessionBus() );
+        if ( fileIndexer.isValid() ) {
+            KConfigGroup group = fileIndexerConfig.group( devGroupName );
+            const QString mountPath = group.readEntry( "mount path", QString() );
+            if( !mountPath.isEmpty() ) {
+                const QStringList folders = group.readPathEntry("folders", QStringList());
+                foreach( const QString& folder, folders ) {
+                    QString path = mountPath + folder;
+                    fileIndexer.indexFolder( path, true, false );
+                }
+            }
+        }
+
+        index = 1;
     }
 
     const bool indexNewlyMounted = fileIndexerConfig.group( "RemovableMedia" ).readEntry( "index newly mounted", false );
     const bool askIndividually = fileIndexerConfig.group( "RemovableMedia" ).readEntry( "ask user", false );
 
     if( index == 0 && indexNewlyMounted && !askIndividually ) {
-        index = 1;
-    }
+        KConfigGroup deviceGroup = fileIndexerConfig.group( "Device-" + entry->url() );
+        deviceGroup.writeEntry( "folders", QStringList() << entry->mountPath() );
 
-    // index automatically
-    if( index == 1 ) {
-        kDebug() << "Device configured for automatic indexing. Calling the file indexer service.";
-        org::kde::nepomuk::FileIndexer fileIndexer( "org.kde.nepomuk.services.nepomukfileindexer", "/nepomukfileindexer", QDBusConnection::sessionBus() );
-        if ( fileIndexer.isValid() ) {
-            fileIndexer.indexFolder( entry->mountPath(), true /* recursive */, false /* no forced update */ );
-        }
+        index = 1;
     }
 
     // ask the user if we should index
@@ -369,11 +402,6 @@ void Nepomuk2::FileWatch::slotDeviceMounted(const Nepomuk2::RemovableMediaCache:
         (new RemovableDeviceIndexNotification(entry, this))->sendEvent();
     }
 
-    else {
-        // TODO: remove all the indexed info
-        kDebug() << "Device configured to not be indexed.";
-    }
-
     //
     // Install the watches
     //
